
WAGTAIL_SYNOPSIS_SYNOPSES_QUERY_FORM_CLASSNAME = ''
WAGTAIL_SYNOPSIS_SYNOPSES_LAYOUT_BLOCK_CLASSNAME = 'curated-content'
WAGTAIL_SYNOPSIS_SYNOPSIS_LIST_BLOCK_CLASSNAME = 'curated-content'
WAGTAIL_SYNOPSIS_SYNOPSIS_LIST_CONTENT_WRAPPER_CLASSNAME = ''

WAGTAIL_SYNOPSIS_TAG_CLASSNAME = 'synopsis-tag'
WAGTAIL_SYNOPSIS_NAMES_CLASSNAME = 'abstract-detail abstract-byline'
